unit UMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, Grids, ExtCtrls, commctrl, StrUtils, Menus,
  Buttons, ShellApi;

Const BatchFileName = 'Batch.txt';

type
  TMainForm = class(TForm)
    FileGrid: TStringGrid;
    StatusBar: TStatusBar;
    Panel1: TPanel;
    AddFileBtn: TButton;
    ProgressBar: TProgressBar;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    AbortBtn: TButton;
    PopupMenu: TPopupMenu;
    PMRemove: TMenuItem;
    PMClearAll: TMenuItem;
    MergeBtn: TBitBtn;
    LinkText: TLabel;
    procedure FormShow(Sender: TObject);
    procedure AddFileBtnClick(Sender: TObject);
    procedure MergeBtnClick(Sender: TObject);
    procedure AbortBtnClick(Sender: TObject);
    procedure PMClearAllClick(Sender: TObject);
    procedure PMRemoveClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormConstrainedResize(Sender: TObject; var MinWidth, MinHeight,
      MaxWidth, MaxHeight: Integer);
    procedure LinkTextMouseEnter(Sender: TObject);
    procedure LinkTextMouseLeave(Sender: TObject);
    procedure LinkTextClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    CancelMerge : Boolean;
    procedure DoMerge(ExitAfter:Boolean);
    procedure AcceptFiles( var msg : TMessage );
      message WM_DROPFILES;
    procedure AddFile(FileName: String);
    procedure AddDir(Directory: String);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

procedure ShellOpen(const Url: string; const Params: string = '');
begin
  ShellAPI.ShellExecute(0, 'Open', PChar(Url), PChar(Params), nil, SW_SHOWNORMAL);
end;

procedure TMainForm.AcceptFiles(var msg : TMessage);
const cnMaxFileNameLen = 255;
var
  i, nCount : integer;
  acFileName : array [0..cnMaxFileNameLen] of char;
  tSearch : TSearchRec;
begin
  nCount := DragQueryFile(msg.WParam, $FFFFFFFF, acFileName, cnMaxFileNameLen);
  for i := 0 to nCount-1 do
  begin
    DragQueryFile(msg.WParam, i, acFileName, cnMaxFileNameLen);

    if FindFirst(acFileName, faAnyFile, tSearch)=0 then
    begin
      if (tSearch.Attr And faDirectory)=faDirectory then
        AddDir(acFileName)
      else
        AddFile(acFileName);
    end;
    FindClose(tSearch);

  end;
  DragFinish(msg.WParam);
end;

function GetFileSize(FileName: String): Int64;
var myFile: THandle;
    myFindData: TWin32FindData;
begin
  Result := 0;
  myFile := FindFirstFile(PChar(FileName), myFindData);
  if (myFile <> INVALID_HANDLE_VALUE) then
  begin
    Windows.FindClose(myFile);
    Int64Rec(Result).Lo := myFindData.nFileSizeLow;
    Int64Rec(Result).Hi := myFindData.nFileSizeHigh;
  end;
end;

procedure TMainForm.AbortBtnClick(Sender: TObject);
begin
  CancelMerge := True;
end;

procedure TMainForm.AddDir(Directory: string);
var tSearch : TSearchRec;
begin
  if FindFirst(IncludeTrailingPathDelimiter(Directory)+'*.vob', faAnyFile, tSearch)=0 then
  begin
    repeat
      AddFile(IncludeTrailingPathDelimiter(Directory)+tSearch.Name);
    until FindNext(tSearch)<>0;
    FindClose(tSearch);
  end;
end;

procedure TMainForm.AddFile(FileName:String);
begin
  if GetFileSize(FileName)>0 then
  begin
    If FileGrid.RowCount>2 Then
      FileGrid.RowCount := FileGrid.RowCount+1
    else
      if FileGrid.Cells[1,1]<>'' then
        FileGrid.RowCount := FileGrid.RowCount+1;

    FileGrid.Cells[1,FileGrid.RowCount-1] := FileName;
    FileGrid.Cells[2,FileGrid.RowCount-1] := inttostr(round(GetFileSize(FileName)/1024/1024))+' MB';
  end;

  if (FileGrid.Cells[1,1]<>'') and (FileGrid.Cells[2,1]<>'') then
    MergeBtn.Enabled := True
  else
    MergeBtn.Enabled := False;
end;

procedure TMainForm.FormActivate(Sender: TObject);
var batchfile,filename:string;
    f : textfile;
    mergenow : boolean;
begin
  mergenow := false;
  batchfile := IncludeTrailingPathDelimiter(ExtractFilePath(Application.ExeName))+BatchFileName;
  if FileExists(batchfile) then
  begin
    AssignFile(f,batchfile);
    Reset(f);
    repeat
      readln(f,filename);
      if Pos('Input: ', filename)=1 then
      begin
        addfile(Copy(filename, length('Input: ')+1, Length(filename)-length('Input: ')));
      end;
      if Pos('Output: ', filename)=1 then
      begin
        SaveDialog.FileName := Copy(filename, length('Output: ')+1, Length(filename)-length('Output: '));
        mergenow := true;
      end;
    until eof(f);
    closefile(f);

    if mergenow=true then DoMerge(True);

    Erase(f);
  end;
end;

procedure TMainForm.FormConstrainedResize(Sender: TObject; var MinWidth,
  MinHeight, MaxWidth, MaxHeight: Integer);
begin
  MinWidth := 500;
  MinHeight := 200;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  DragAcceptFiles( Handle, True );
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
  LinkText.Top := Panel1.Height-24;
  LinkText.Left := 8;
end;

procedure TMainForm.FormShow(Sender: TObject);
var r : TRect;
begin
  Statusbar.perform(SB_GETRECT,0,integer( @R ));
  progressbar.parent := Statusbar;
  progressbar.top := r.top;
  progressbar.left := r.left;
  progressbar.width := (r.right-r.left);
  progressbar.height := (r.bottom-r.top);
  FileGrid.Cells[1,0] := 'Filename';
  FileGrid.Cells[2,0] := 'Filesize';
end;

procedure TMainForm.LinkTextClick(Sender: TObject);
begin
  ShellOpen(LinkText.Caption);
end;

procedure TMainForm.LinkTextMouseEnter(Sender: TObject);
begin
  LinkText.Font.Style := [fsUnderline];
  LinkText.Font.Color := clMenuHighlight;
end;

procedure TMainForm.LinkTextMouseLeave(Sender: TObject);
begin
  LinkText.Font.Style := [];
  LinkText.Font.Color := clMenuText;
end;

procedure TMainForm.DoMerge(ExitAfter:Boolean);
var y:integer;
    Output, Input : file;
    NR, NW: Integer;
    Buf: Array[0..High(Word)*4] of Char;
    TotalSize : Int64;
    BytesWritten: Int64;
    Percent: Byte;
begin
  TotalSize := 0;
  BytesWritten := 0;

  AssignFile(Output, SaveDialog.FileName);
  ReWrite(Output, 1);

  for y:=1 to FileGrid.RowCount-1 do
    Inc(TotalSize,StrToInt(leftstr(FileGrid.Cells[2,y], Length(FileGrid.Cells[2,y])-3)));

  AbortBtn.Visible := True;
  MergeBtn.Visible := False;
  AbortBtn.Left := MergeBtn.Left;
  AbortBtn.Top := MergeBtn.Top;
  CancelMerge := False;
  AddFileBtn.Enabled := False;
  FileGrid.Enabled := False;

  for y:=1 to FileGrid.RowCount-1 do
  begin
    AssignFile(Input, FileGrid.Cells[1,y]);
    Reset(Input, 1);

    FileGrid.Row := y;

    Repeat
      Application.ProcessMessages;
      System.BlockRead(Input, Buf, SizeOf(Buf), NR);
      System.BlockWrite(Output, Buf, NR, NW);
      Inc(BytesWritten, NW);
      Percent := Round( ((BytesWritten/1024/1024)/TotalSize) * 100 );
      StatusBar.Panels[1].Text := IntToStr(Round(BytesWritten/1024/1024))+'/'+IntToStr(TotalSize)+' MB ('+IntToStr(Percent)+'%)';
      ProgressBar.Position := Percent;
    Until (NR=0) Or (NR<>NW) Or (CancelMerge=True);

    if CancelMerge then Break;

    CloseFile(Input);
  end;

  FileGrid.Enabled := True;
  AddFileBtn.Enabled := True;
  MergeBtn.Visible := True;
  AbortBtn.Visible := False;

  ProgressBar.Position := 0;
  if CancelMerge then
    StatusBar.Panels[1].Text := 'Aborted by user.'
  else
    StatusBar.Panels[1].Text := 'File successfully created: '+ExtractFileName(SaveDialog.FileName);

  CloseFile(Output);

  if ExitAfter then MainForm.Close;

end;

procedure TMainForm.MergeBtnClick(Sender: TObject);
begin
  If SaveDialog.Execute() then
  begin
    DoMerge(False);
  end;
end;

procedure TMainForm.AddFileBtnClick(Sender: TObject);
var i:integer;
begin
  if OpenDialog.Execute() then
    for i := 0 to OpenDialog.Files.Count - 1 do
      begin
        AddFile(OpenDialog.Files.Strings[i]);
      end;
end;

procedure TMainForm.PMClearAllClick(Sender: TObject);
var x,y:integer;
begin
  for y:=FileGrid.RowCount downto 1 do
    for x:=0 to FileGrid.ColCount do
      FileGrid.Cells[x,y] := '';
  FileGrid.RowCount := 2;
  MergeBtn.Enabled := False;
end;

procedure TMainForm.PMRemoveClick(Sender: TObject);
var y:integer;
begin
  for y:= FileGrid.Row to FileGrid.RowCount-2 do
  begin
    FileGrid.Cells[1,y] := FileGrid.Cells[1,y+1];
    FileGrid.Cells[2,y] := FileGrid.Cells[2,y+1];
  end;
  If FileGrid.RowCount>2 then
    FileGrid.RowCount := FileGrid.RowCount -1
  else
  begin
    FileGrid.Cells[1,1] := '';
    FileGrid.Cells[2,1] := '';
  end;

  if (FileGrid.Cells[1,1]<>'') and (FileGrid.Cells[2,1]<>'') then
    MergeBtn.Enabled := True
  else
    MergeBtn.Enabled := False;
end;

end.
